import os from "os";
import fs from "fs";
import path from "path";
import { C } from "./common";

const URLS = {
	GITHUB: `https://github.com/kangkang520/node-addons/releases/download/cv{{version}}/cv_{{platform}}_{{arch}}.node`,
	URNAS: `http://git.urnas.cn:5200/yizhi-js-lib/opencv/releases/download/{{version}}/cv_{{platform}}_{{arch}}.node`,
}

function getURL(template: string) {
	const version = require("../../package.json").releaseVersion;
	let platform = "";
	let arch = "";
	switch (os.platform()) {
		case "win32":
			platform = "windows";
			break;
		case "linux":
			platform = "linux";
			break;
		default:
			throw new Error(`Unsupported platform: ${os.platform()}, Please compile the addon yourself.`);
	}
	switch (os.arch()) {
		case "x64":
			arch = "x64";
			break;
		default:
			throw new Error(`Unsupported architecture: ${os.arch()}, Please compile the addon yourself.`);
	}

	return template.replaceAll("{{version}}", version).replaceAll("{{platform}}", platform).replaceAll("{{arch}}", arch);
}

async function getStream() {
	for (const [name, url] of Object.entries(URLS)) {
		try {
			return await fetch(getURL(url)).then(res => {
				if (res.status != 200) throw new Error("Failed to download addon.");
				return res.blob().then(b => b.stream());
			})
		} catch (e) { }
	}
	throw new Error("Failed to download addon.");
}

export async function downloadAddon(savename?: string) {
	const defaultAddon = path.resolve(process.cwd(), C("ADDON_PATH"));
	const saveName = savename ? path.resolve(path.dirname(defaultAddon), savename) : defaultAddon;
	if (fs.existsSync(saveName)) return saveName;

	await fs.promises.mkdir(path.dirname(saveName), { recursive: true });

	const stream = await getStream();
	const cacheFile = await new Promise<string>((resolve, reject) => {
		const cacheFile = path.join(os.tmpdir(), Date.now() + ".cv.node");
		let fsStream!: ReturnType<typeof fs.createWriteStream>;
		stream.pipeTo(new WritableStream({
			start(controller) {
				fsStream = fs.createWriteStream(cacheFile);
			},
			async write(chunk, controller) {
				await new Promise<void>((resolve, reject) => fsStream.write(chunk, err => err ? reject(err) : resolve()));
			},
			close() {
				fsStream.end();
				resolve(cacheFile);
			},
			abort() { }
		})).catch(reject);
	});

	await fs.promises.cp(cacheFile, saveName);
	await fs.promises.rm(cacheFile);
	return saveName;
}
