import { CVMat } from "./addon";
import { FromCV, M, TypedArray } from "./common";
import { IMREAD_COLOR_BGR } from "./consts";


export class Mat {
	#mat: any
	public constructor();
	public constructor(rows: number, cols: number, type: number, data?: TypedArray);
	public constructor(sizes: number[], type: number, data?: TypedArray);
	public constructor(va?: any, vb?: any, vc?: any, vd?: any) {
		const _CVMat = CVMat();
		if (va instanceof _CVMat) this.#mat = va;
		if (typeof va === "number") this.#mat = new _CVMat([va, vb], vc, vd);
		else if (typeof va === "undefined") this.#mat = new _CVMat();
		else if (va instanceof Array) this.#mat = new _CVMat(va, vb, vc);
	}

	private get __mat__() { return this.#mat; }

	public get empty(): boolean { return this.#mat.IsEmpty(); }
	public get cols(): number { return this.#mat.GetCols(); }
	public get rows(): number { return this.#mat.GetRows(); }
	public get type(): number { return this.#mat.GetType(); }
	public get depth(): number { return this.#mat.GetDepth(); }
	public get channels(): number { return this.#mat.GetChannels(); }
	public get flags(): number { return this.#mat.GetFlags(); }
	public get dims(): number { return this.#mat.GetDims(); }
	public get isContinuous(): boolean { return this.#mat.GetIsContinuous(); }
	public get isSubmatrix(): boolean { return this.#mat.GetIsSubmatrix(); }
	public get elemSize(): number { return this.#mat.GetElemSize(); }
	public get elemSize1(): number { return this.#mat.GetElemSize1(); }
	public total(startDim?: number, endDim?: number): number {
		if (startDim === null || startDim === undefined) return this.#mat.GetTotal();
		else return this.#mat.GetTotalWithDim(startDim, endDim ?? 2147483647);
	}
	public get size(): number[] { return this.#mat.GetSize(); }
	public get data(): Uint8Array {
		if (this.isContinuous) return new Uint8Array(this.#mat.GetData());
		else return this.copyTo(this.clone()).data;
	}

	public clone() { return FromCV(this.#mat.Clone()); }
	public copyTo(mat: Mat) { return this.#mat.CopyTo(M(mat)), mat; }

	public row(y: number) { return FromCV(this.#mat.Row(y)); }
	public col(x: number) { return FromCV(this.#mat.Col(x)); }
	public rowRange(start: number, end: number) { return FromCV(this.#mat.RowRange(start, end)); }
	public colRange(start: number, end: number) { return FromCV(this.#mat.ColRange(start, end)); }
	public diag(d: number) { return FromCV(this.#mat.Diag(d)); }
};

export function imread(filename: string, flag?: number) { return FromCV(CVMat().ImRead(filename, flag ?? IMREAD_COLOR_BGR)); }
export function imdecode(data: Uint8Array, flag?: number) { return FromCV(CVMat().ImDecode(data, flag ?? IMREAD_COLOR_BGR)); }
export function imwrite(filename: string, mat: Mat, params?: number[]): boolean { return CVMat().ImWrite(M(mat), filename, params); }
export function imencode(ext: string, mat: Mat, params?: number[]): Uint8Array | null {
	const res = CVMat().ImEncode(M(mat), ext, params);
	if (!res) return null;
	return new Uint8Array(res);
}
